/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

import com.ibm.etools.c.CClassifier;
import com.ibm.etools.c.CDerivableType;
import com.ibm.etools.c.CFactory;
import com.ibm.etools.c.CFunction;
import com.ibm.etools.c.CParameter;
import com.ibm.etools.c.CPointer;
import com.ibm.etools.c.datatypes.CDirectionKind;
import com.ibm.etools.c.datatypes.CVoid;

/**
 * The CFunctionParser class parses the string representation of a function or
 * function pointer and returns the corresponding CFunction or CPointer object.
 */
public class CFunctionParser {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    private String _name;

    private CFactory _factory;

    private CTypeParser _typeParser;

    private CFunction _function;

    private CPointer _funcPointer;

    private boolean _isFunctionPointer;

    /**
     * Create a CFunctionParser object from a given signature string and a model
     * converter
     */
    public CFunctionParser(String name, CModelConverter modelConverter) throws CException {
        _name = name;
        //_factory = modelConverter.getImporterResource().getFactory();
        _factory = CFactory.eINSTANCE;
        _typeParser = modelConverter.getTypeParser();
        _isFunctionPointer = false;
        parse();
    }

    /**
     * Return a CPointer object which represents the function pointer
     */
    public CPointer getPointer() {
        return _funcPointer;
    }

    /**
     * Return a CFunction object which represents a function type.
     */
    public CFunction getFunction() {
        return _function;
    }

    /**
     * Is this a function pointer?
     */
    public boolean isFunctionPointer() {
        return _isFunctionPointer;
    }

    /**
     * Parse the string representation of the function pointer and create a
     * CPointer object for it. This is the method that does the main job.
     */
    private void parse() throws CException {

        // Create an unnamed function to derive the function pointer
        _function = _factory.createCFunction();
        _function.setName("");

        // Locate the function or function pointer signature
        int functionIndex = _name.indexOf(CIM.P_FUNCTION_PTR);
        int paramIndex = 0;
        if (functionIndex >= 0) {
            _isFunctionPointer = true;
            paramIndex = functionIndex + 4;
        } else {
            functionIndex = _name.indexOf(CIM.P_FUNCTION_SIGNATURE);
            paramIndex = functionIndex + 3;
        }

        String returnType = _name.substring(0, functionIndex).trim();

        if (returnType.length() > 0) {
            if (returnType.startsWith("inline "))
                returnType = returnType.substring(7).trim();

            if (returnType.startsWith("static "))
                returnType = returnType.substring(7).trim();

            addParameter(returnType, true);
        }

        int commasIndex = paramIndex;
        String aparameter = null;
        // Extract each parameter from the param list
        while ((commasIndex = _name.indexOf(',', paramIndex)) > 0) {
            aparameter = _name.substring(paramIndex, commasIndex).trim();
            paramIndex = commasIndex + 1;
            addParameter(aparameter, false);
        }

        // Process the last parameter. If the last param is "...", set the
        // IsVarArg attribute to true.
        int lastParenIndex = _name.lastIndexOf(')');
        String lastArg = _name.substring(paramIndex, lastParenIndex).trim();
        if (lastArg.length() > 0) {
            if (lastArg.equals(CIM.P_FUNCTION_VARIABLE_ARGUMENT))
                _function.setIsVarArg(Boolean.TRUE);
            else
                addParameter(lastArg, false);
        }

        // Create a CPointer object if this is a function pointer.
        if (_isFunctionPointer) {
            _funcPointer = _factory.createCPointer();
            _funcPointer.setDerives(_function);
        }
    }

    /**
     * Parse the string representation of a function parameter and create a
     * CParameter object for it, then add the parameter to the function.
     */
    private CParameter addParameter(String paramString, boolean isReturn) throws CException {

        // Return null if the parameter string is empty
        if (paramString == null || paramString.length() == 0) {
            return null;
        }

        String paramName = "";
        String paramType = paramString;

        // A regular parameter may have a name. We need to extract the name from
        // the type.
        // A return parameter only has a type.
        //
        CTypedElementParser typedElement = new CTypedElementParser(paramString);
        paramName = typedElement.getName();
        paramType = typedElement.getType();

        CDerivableType type = null;
        try {
            type = _typeParser.findOrCreateType(paramType, true);
        } catch (Exception e) {
            CResource.writeLog(this, "addParameter", e);
        }

        CParameter aparameter = _factory.createCParameter();
        aparameter.setName(paramName);
        if (type instanceof CClassifier)
            aparameter.setType((CClassifier) type);

        // Set the direction to indicate whether this is a regular argument or a
        // return type.
        if (isReturn) {
            // **TO FIX**
            //aparameter.setDirection(CIM.M_DIRECTION_RETURN);
            aparameter.setDirection(CDirectionKind.RETURN_LITERAL);
        } else {
            // **TO FIX**
            //aparameter.setDirection(CIM.M_DIRECTION_ARGUMENT);
            aparameter.setDirection(CDirectionKind.PARAMETER_LITERAL);
        }

        if (CPreferenceStore.isPopulateTypeDescriptorModel() && !(type instanceof CVoid)) {
            aparameter.setInstanceTDBase(CTypeDescriptorBuilder.createInstanceForVariable(aparameter));
        }
        _function.getParameter().add(aparameter);

        return aparameter;
    }

}