/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

import java.util.HashMap;

import com.ibm.etools.c.CClassifier;
import com.ibm.etools.c.CDatatype;
import com.ibm.etools.c.datatypes.CChar;
import com.ibm.etools.c.datatypes.CDouble;
import com.ibm.etools.c.datatypes.CFloat;
import com.ibm.etools.c.datatypes.CInt;
import com.ibm.etools.c.datatypes.CLongDouble;
import com.ibm.etools.c.datatypes.CLongLong;
import com.ibm.etools.c.datatypes.CSignedChar;
import com.ibm.etools.c.datatypes.CUnsignedChar;
import com.ibm.etools.c.datatypes.CUnsignedInt;
import com.ibm.etools.c.datatypes.CUnsignedLong;
import com.ibm.etools.c.datatypes.CUnsignedLongLong;
import com.ibm.etools.c.datatypes.CUnsignedShort;
import com.ibm.etools.c.datatypes.CWchar;
import com.ibm.etools.c.datatypes.DatatypesFactory;
import com.ibm.etools.typedescriptor.BaseTDType;

/**
 * The CPrimitiveTypesMapper class provides two mappings for C primitive types:
 * 1) String - CDatatype, which maps a string representation of a C primitive
 * type to a C model type object. 2) CDatatype - BaseTDType, which maps a C
 * model type to the corresponding Type Descriptor type.
 */

public class CPrimitiveTypesMapper {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    private static CPrimitiveTypesMapper _instance = new CPrimitiveTypesMapper();

    private static DatatypesFactory _factory;

    // The hash table for mappings between the string and model representation
    // of C primitive types.
    private static HashMap _map;

    // The hash table for mappings between C model types and TypeDescriptor
    // types.
    private static HashMap _tdmap;

    /**
     * Create a CPrimitiveTypesMapper object and initialize the two hash tables.
     */
    private CPrimitiveTypesMapper() {
        //_factory = CImporter.instance().getDatatypesFactory();
        _factory = DatatypesFactory.eINSTANCE;
        _map = new HashMap(60);
        _tdmap = new HashMap(12);

        CInt int_type = _factory.createCInt();
        _map.put("int", int_type);
        _map.put("signed int", int_type);
        _map.put("int signed", int_type);
        _map.put("signed", int_type);

        CChar char_type = _factory.createCChar();
        _map.put("char", char_type);

        com.ibm.etools.c.datatypes.CShort shortType = _factory.createCShort();
        _map.put("short", shortType);
        _map.put("short int", shortType);
        _map.put("int short", shortType);
        _map.put("signed short", shortType);
        _map.put("short signed", shortType);
        _map.put("signed short int", shortType);
        _map.put("short signed int", shortType);
        _map.put("int short signed", shortType);
        _map.put("int signed short", shortType);

        com.ibm.etools.c.datatypes.CLong longType = _factory.createCLong();
        _map.put("long", longType);
        _map.put("long int", longType);
        _map.put("int long", longType);
        _map.put("signed long", longType);
        _map.put("long signed", longType);
        _map.put("signed long int", longType);
        _map.put("long int signed", longType);
        _map.put("long signed int", longType);

        CLongLong longlongType = _factory.createCLongLong();
        _map.put("long long", longlongType);
        _map.put("long long int", longlongType);
        _map.put("signed long long", longlongType);
        _map.put("long long signed", longlongType);
        _map.put("signed long long int", longlongType);
        _map.put("long long int signed", longlongType);
        _map.put("long long signed int", longlongType);

        // unsigned and unsigned int are the same thing.
        CUnsignedInt unsignedInt_type = _factory.createCUnsignedInt();
        _map.put("unsigned int", unsignedInt_type);
        _map.put("unsigned", unsignedInt_type);
        _map.put("int unsigned", unsignedInt_type);

        CUnsignedShort unsignedShortType = _factory.createCUnsignedShort();
        _map.put("unsigned short", unsignedShortType);
        _map.put("short unsigned", unsignedShortType);
        _map.put("unsigned short int", unsignedShortType);
        _map.put("unsigned int short", unsignedShortType);
        _map.put("short int unsigned", unsignedShortType);
        _map.put("short unsigned int", unsignedShortType);
        _map.put("int short unsigned", unsignedShortType);
        _map.put("int unsigned short", unsignedShortType);

        CUnsignedLong unsignedLongType = _factory.createCUnsignedLong();
        _map.put("unsigned long", unsignedLongType);
        _map.put("long unsigned", unsignedLongType);
        _map.put("unsigned long int", unsignedLongType);
        _map.put("long int unsigned", unsignedLongType);
        _map.put("int long unsigned", unsignedLongType);
        _map.put("long unsigned int", unsignedLongType);
        _map.put("int unsigned long", unsignedLongType);

        CUnsignedLongLong unsignedLongLongType = _factory.createCUnsignedLongLong();
        _map.put("unsigned long long", unsignedLongLongType);
        _map.put("long long unsigned", unsignedLongLongType);
        _map.put("unsigned long long int", unsignedLongLongType);
        _map.put("long long int unsigned", unsignedLongLongType);
        _map.put("long long unsigned int", unsignedLongLongType);

        CUnsignedChar unsignedCharType = _factory.createCUnsignedChar();
        _map.put("unsigned char", unsignedCharType);
        _map.put("char unsigned", unsignedCharType);

        CSignedChar signedCharType = _factory.createCSignedChar();
        _map.put("signed char", signedCharType);
        _map.put("char signed", signedCharType);

        CWchar wcharType = _factory.createCWchar();
        _map.put("wchar_t", wcharType);

        _map.put("void", _factory.createCVoid());

        CFloat floatType = _factory.createCFloat();
        _map.put("float", floatType);

        CDouble doubleType = _factory.createCDouble();
        _map.put("double", doubleType);

        CLongDouble longDoubleType = _factory.createCLongDouble();
        _map.put("long double", longDoubleType);
    }

    /**
     * Return the only instance of CPrimitiveTypesMapper
     */
    public static CPrimitiveTypesMapper instance() {
        return _instance;
    }

    /**
     * Create the Type Descriptor maps between C model types and Type Descriptor
     * types
     */
    public static void createTypeDescriptorMap() {
        _tdmap.clear();
        _tdmap.put(getTypeFromName("int"), CTypeDescriptorBuilder.createInt());
        _tdmap.put(getTypeFromName("char"), CTypeDescriptorBuilder.createChar());
        _tdmap.put(getTypeFromName("short"), CTypeDescriptorBuilder.createShort());
        _tdmap.put(getTypeFromName("long"), CTypeDescriptorBuilder.createLong());
        _tdmap.put(getTypeFromName("long long"), CTypeDescriptorBuilder.createLongLong());
        _tdmap.put(getTypeFromName("unsigned int"), CTypeDescriptorBuilder.createUnsignedInt());
        _tdmap.put(getTypeFromName("unsigned short"), CTypeDescriptorBuilder.createUnsignedShort());
        _tdmap.put(getTypeFromName("unsigned long"), CTypeDescriptorBuilder.createUnsignedLong());
        _tdmap.put(getTypeFromName("unsigned long long"), CTypeDescriptorBuilder.createUnsignedLongLong());
        _tdmap.put(getTypeFromName("unsigned char"), CTypeDescriptorBuilder.createUnsignedChar());
        _tdmap.put(getTypeFromName("signed char"), CTypeDescriptorBuilder.createSignedChar());
        _tdmap.put(getTypeFromName("wchar_t"), CTypeDescriptorBuilder.createWchar());
        _tdmap.put(getTypeFromName("float"), CTypeDescriptorBuilder.createFloat());
        _tdmap.put(getTypeFromName("double"), CTypeDescriptorBuilder.createDouble());
        _tdmap.put(getTypeFromName("long double"), CTypeDescriptorBuilder.createLongDouble());
    }

    /**
     * Does the string represent a C primitive type?
     */
    public static boolean isPrimitiveType(String type) {

        return _map.containsKey(type);
    }

    /**
     * Return the corresponding C model object from the type string
     */
    public static CDatatype getTypeFromName(String name) {
        return (CDatatype) _map.get(name);
    }

    /**
     * Create a mapping between a C model type and a Type Descriptor type in the
     * TD map.
     */
    public static void putTDType(CClassifier ctype, BaseTDType tdtype) {
        _tdmap.put(ctype, tdtype);
    }

    /**
     * Return the corresponding TypeDescriptor type for the given C type.
     */
    public static BaseTDType getTDTypeFromCType(CClassifier ctype) {
        return (BaseTDType) _tdmap.get(ctype);
    }

}