// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.cobol.importer;


import java.util.*;

/**
 *   The class 'Record' is an abstract base class which encapsulates the
 *   behaviour and data associated with a single COBOL record from the CICS
 *   transaction.  This class is designed to store information obtained from
 *   the parse which is necessary to create the JavaBeans required to support
 *   CICS/ECI calls to the original CICS transaction.  An instance of one of the
 *   classes derived from 'Record' is created by the parser as each new record
 *   declaration is encountered in the COBOL program.
 */

abstract public class CobolRecord 
{
	private static String copyright = "(c) Copyright IBM Corporation 1998, 1999.";

   protected String name_;       // COBOL record's name
   protected String redefinedItem_; // Name of elementary item being redefined
   protected String value_;
   protected String figConst_;
   protected String picString_;  // COBOL record's picture string definition
   protected CobolRecord owner_; // Parent group of current COBOL record
   protected Vector dependedOnItem_; // Vector containing qualified name of object of ODO
   protected Vector list_;       // Contains a list of elementary records plus
								 // any sub-groups contained withing current
								 // COBOL record
   protected boolean filler_;    // Indicates if COBOL record is designated
								 // as 'filler' in the COBOL program
   protected boolean alignment_; // Indicates if current COBOL record is
								 // desginated as 'sync' in the COBOL program
   protected int level_;         // COBOL record's level
   protected int usage_;         // Indicates COBOL record's usage
   protected int sign_;          // Indicates COBOL record's sign
   protected int arrayBound_;    // Indicates COBOL record's array size
   protected int arrayLowBound;  /*26260 GIM Save lower bound for ODO */
   protected Class javaType_;
   protected String [] qualifiedName_; // Fully qualified name of record
   protected String cobolTypeString_;  
   protected Vector varLengthArrayNames_; // Arrays whose size depend on record's value
   protected Vector redefineTypes_;  // Records that redefine this record.
   protected Vector keys_; 		// Contains the keys
   protected Vector possibleValues_; // Contains the possible values for the keys 

   // list of usage values
   public static final short NOUSAGE           = 0;
   public static final short COMP              = 1;
   public static final short COMP1             = 2;
   public static final short COMP2             = 3;
   public static final short COMP3             = 4;
   public static final short COMP5             = 5;
   public static final short DISPLAY           = 6;
   public static final short DISPLAY1          = 7;
   public static final short NATIONAL          = 8;
   // list of sign values
   public static final short NOSIGN            = 9;
   public static final short TRAILING          = 10;
   public static final short LEADING           = 11;
   public static final short TRAILINGSEPARATE  = 12;
   public static final short LEADINGSEPARATE   = 13;




   // The ctor takes the owning COBOL group, the COBOL record name and the level
   // as arguments
   CobolRecord(CobolRecord owner, String name, int level)
   {
	  owner_ = owner;
	  level_ = level;
	  filler_ = false;
	  alignment_ = false;
	  list_ = new Vector();
	  arrayBound_ = 1;
	  arrayLowBound = 0;
	  usage_ = NOUSAGE;
	  sign_ = NOSIGN;
	  name_ = name;
	  value_ = null;
	  figConst_ = null;
	  redefinedItem_ = null;
	  dependedOnItem_ = null;
	  redefineTypes_ = new Vector();
	  varLengthArrayNames_ = new Vector();
	  possibleValues_ = new Vector();
	  keys_ = new Vector();
   }   
   void addRedefineType(CobolRecord rec)
   {
	  redefineTypes_.addElement(rec);
	  return;
   }   
   int arrayBound()
   {
	  return arrayBound_;
   }   
   String cobolType() {

	  String sign = new String();
	  String cobolType = new String();
	  int usageType = NOUSAGE;
	  int signType = NOSIGN;

	  if (this instanceof CobolGroupRecord) {
		 usageType = ((CobolGroupRecord)this).usage();
		 signType = ((CobolGroupRecord)this).sign();
	  }
	  else if (this instanceof CobolElementaryRecord) {
		 usageType = ((CobolElementaryRecord)this).usage();
		 signType = ((CobolElementaryRecord)this).sign();
	  }

	  switch (signType) {
		 case NOSIGN:
			break;
		 case LEADING:
			sign = "SIGN_LEADING";
			break;
		 case TRAILING:
			sign = "SIGN_TRAILING";
			break;
		 case LEADINGSEPARATE:
			sign = "SIGN_LEADING_SEPARATE";
			break;
		 case TRAILINGSEPARATE:
			sign = "SIGN_TRAILING_SEPARATE";
			break;
	  }

	  switch (usageType) {
		 case NOUSAGE:
		 case DISPLAY:
			if (signType == NOSIGN)
			   cobolType = "DISPLAY";
			else
			   cobolType = sign;
			break;
		 case COMP:
			if (signType == NOSIGN)
			   cobolType = "COMP";
			else
			   cobolType = sign;
			break;
		 case COMP1:
			if (signType == NOSIGN)
			   cobolType = "COMP1";
			else
			   cobolType = sign;
			break;
		 case COMP2:
			if (signType == NOSIGN)
			   cobolType = "COMP2";
			else
			  cobolType = sign;
			break;
		 case COMP3:
			if (signType == NOSIGN)
			   cobolType = "COMP3";
			else
			   cobolType = sign;
			break;

		 /***
		  *** JJS 10-24-2001
		  ***/
		 case COMP5:
			if (signType == NOSIGN)
			   cobolType = "COMP5";
			else
			   cobolType = sign;
			break;
		 case DISPLAY1:
			cobolType = "DISPLAY1";
			break;
		 case NATIONAL:
		 	cobolType = "NATIONAL";
		 	break; 
	  }

	  return cobolType;

   }      
   CobolRecord element(int index) {

	  return (CobolRecord)list_.elementAt(index);

   }   
   int level()
   {
	  return level_;
   }   
   Vector list()
   {
	  return list_;
   }   
   String name()
   {
	  return name_;
   }   
   CobolRecord owner()
   {
	  return owner_;
   }   
   String [] qualifiedName()
   {
	  return qualifiedName_;
   }   
   String redefinedItem()
   {
	  return redefinedItem_;
   }   
/**
 * Removes the commarea name from the array names in varLengthArrayNames_.
 * Creation date: (4/6/99 9:26:28 AM)
 * @return java.util.Vector
 * @param v java.util.Vector
 */
public Vector removeCommareaName() {
	Vector newVec = new Vector();
	for (int i=0; i < varLengthArrayNames_.size(); i++)
	{
		String [] fullName = (String[]) varLengthArrayNames_.elementAt(i);
		String [] name = new String[fullName.length-1];
		for (int j=0; j < fullName.length-1; j++)
		{
			name[j]=fullName[j+1];
		}
		newVec.addElement(name);
	}
	return newVec;
}
/**
 * This method was created in VisualAge.
 * @return boolean - true if fullName and partName are the same item
 * @param fullName java.lang.String[] - fully qualified name of item
 * @param partName java.util.Vector
 */
 boolean sameItem(String[] fullName, Vector partName) {
	if (partName.size() > fullName.length){
		return false;
	}
	if (!fullName[fullName.length-1].equals(partName.firstElement())){
		return false;
	}
	int j=0;
	for (int i=partName.size()-1; i>=0; i--){
		while (!fullName[j].equalsIgnoreCase((String)partName.elementAt(i))){
			j++;
			if (j == fullName.length){
				return false;
			}
		}
	}	
		
	return true;
}
   void setAlignment(boolean alignment)
   {
	  alignment_ = alignment;
   }   
   void setArrayBound(int bound)
   {
	  arrayBound_ = bound;
   }   
/**
 * This method was created in VisualAge.
 * @param str java.lang.String
 */
 void setDependedOnItem(Vector vec) {
	 dependedOnItem_ = vec;
}
   void setFigConst(String figConst) {

	  figConst_ = figConst;

   }   
   void setFiller(boolean filler)
   {
	  filler_ = filler;
   }   
   void setName(String name)
   {
	  name_ = name;
   }   
   void setNewPicClass(Class javaType)
   {
	  javaType_ = javaType;
   }   
   void setPicString(String picString)
   {
	  picString_ = picString;
   }   
   void setPossibleValues(String key, Vector possibleValues)
   {
	   Object[] possibleVals = new Object[possibleValues.size()];
	   possibleValues.copyInto(possibleVals);
	   possibleValues_.addElement(possibleVals);
	   keys_.addElement(key);
   }   
   void setRedefinedItem(String str)
   {
	  redefinedItem_ = str;
   }   
   void setSign(short value) {

		 sign_ = value;

   }   
   void setUsage(short value) {

	  usage_ = value;

   }   
   void setValue(String value) {

	  value_ = value;

   }   
   void setVarLengthArrayName(String [] name)
   {
	  varLengthArrayNames_.addElement(name);
   }   
  /**
   * @return
   */
	/*26260 GIM Save lower bound for ODO */
  public int getArrayLowBound() {
    return arrayLowBound;
  }

  /**
   * @param i
   */
	/*26260 GIM Save lower bound for ODO */
  public void setArrayLowBound(int i) {
    arrayLowBound = i;
  }

}
